<?php

namespace Scriptor\Modules\MinResModule;

use Imanager\Category;
use Imanager\Item;
use Imanager\Util;
use Scriptor\Core\Helper;
use Scriptor\Core\Module;
use Scriptor\Core\Page;
use Scriptor\Core\Scriptor;
use Scriptor\Core\Site;

/**
 * MinResModule Class
 *
 * MinRes class to manage the MinRes pages 
 * in the Scriptor editor area.
 *
 */
class MinResModule extends Module
{
	/**
	 * Module version 
	 * 
	 * Use ~~ echo MinResModule::VERSION; ~~~
	 * to get the current module version
	 */
	const VERSION = '1.5.1';

	public Site $site;

	/**
	 * Template defaults
	 * 
	 * 0 = Unlimited
	 */
	private $templates = 
	[
		'hero' => [
			'count' => 1,
		],
		'skills' => [
			'count' => 0
		],
		'interests' => [
			'count' => 0
		],
		'achievements' => [
			'count' => 0
		],
		'timeline' => [
			'count' => 6
		],
		'footer' => [
			'count' => 1
		]
	];

	/**
	 * Current page or null
	 * 
	 * @var object|null
	 */
	//private $page;

	//private $pages;

	private $minresConfig = [];

	private static $initialized = false;

    public static function moduleInfo() : array
    {
        return [
            'name' => 'MinResModule',
            'description' => 'MinRes - A Minimal HTML & CSS Resume Module and Theme for Scriptor CMS. During installation, the Mager module generates several special pages. <a href="https://scriptor-cms.info/extensions/extension-themes/minres-minimal-resume-theme/">More information</a>',
            'version' => self::VERSION,
			'position' => 100,
            'author' => 'Bigin',
            'author_website' => 'https://ehret-studio.com',
			'template' => [
				'meta' => [
					'title' => 'Resume of Abbie Bradley',
					'description' => 'Resume website of Abbie Bradley, web developer.',
					'keywords' => 'resume, cv, development, design, web application',
				],
				// Allowed template names to run in this app
				'allowed' => [
					'home'
				],
				// Define cacheable template names
				'cacheable' => [
					'home'
				]
			]
        ];
    }

	public static function moduleHooks() : array
	{
		return [
			'Pages::beforeRenderEditorPage' => [
				[
					'module' => 'MinResModule',
					'method' => 'renderSelector'
				],
				[
					'module' => 'MinResModule',
					'method' => 'renderEditorPage'
				]
			],
			'Pages::afterCheckAction' => [
				[
					'module' => 'MinResModule',
					'method' => 'checkAction'
				]
			]
		];
	}

	public function install() : bool
	{
		$this->init();
		$pageTypeExists = $this->site->pages()->getPages('pagetype=minres');
		if (!$pageTypeExists) {
			$token = Util::randomToken();
			$_SESSION['MinRes']['token'] = $token;
			$_SESSION['MinRes']['msgs'] = $this->msgs;
			unset($_SESSION['msgs']);
			$this->redirect($this->siteUrl.'/MinResModule/'.$token);
		}
		return false;
	}

	public function uninstall() : bool
	{
		$this->init();
		$pages = $this->site->pages()->getPages('pagetype=minres');
		if ($pages) {
			foreach ($pages as $page) {
				try {
					$this->site->pages()->deletePage($page);
				} catch (\ErrorException $e) {
					echo 'Exception: ' . $e->getMessage();
				}
			}
		}
		return true;
	}

	public function init()
	{
		if (!self::$initialized) {
			parent::init();
			$this->site = Scriptor::getSite();
			$this->csrf = Scriptor::getCSRF();
			//$this->pages = $this->imanager->getCategory('name=Pages');
			if (isset($this->config['modules']['MinResModule'])) {
				$this->minresConfig = $this->config['modules']['MinResModule'];
				$this->config['template'] = $this->minresConfig['template'];
			}
			$this->addResource('link', [
				'rel' => 'stylesheet',
				'href' => dirname($this->siteUrl).'/site/modules/MinResModule/css/styles.css'
			]);

			self::$initialized = true;
		}
	}

	public function execute() 
	{
		$this->checkAction();

		$token = $this->segments->getLast();
		if (!isset($_SESSION['MinRes']['token']) || $token !== $_SESSION['MinRes']['token']) {
			$this->addMsg('error', Helper::rawClassName(__CLASS__).': '.$this->i18n['error_minres_install']);
			$this->redirect($this->siteUrl.'/MinResModule/'.$token);
		}

		$this->pageTitle = 'MinRes Pages Install';
		$this->pageContent = $this->renderInstaller($token);
		$this->breadcrumbs = '<li><span>'.$this->i18n['minres_menu'].'</span></li>';
	}

	public function ___checkAction()
	{
		//sleep(5);
		if($this->input->post->action == 'save-hero-page') {
			if($this->saveHeroPage($event->object)) {
				$this->redirect("./?page={$this->page->id}");
			}
		} elseif($this->input->post->action == 'save-skills-page') {
			if($this->saveSkillsPage($event->object)) {
				$this->redirect("./?page={$this->page->id}");
			}
		} elseif($this->input->post->action == 'save-interests-page') {
			if($this->saveSkillsPage($event->object)) {
				$this->redirect("./?page={$this->page->id}");
			}
		} elseif($this->input->post->action == 'save-achievements-page') {
			if($this->saveAchievementsPage($event->object)) {
				$this->redirect("./?page={$this->page->id}");
			}
		} elseif($this->input->post->action == 'save-timeline-page') {
			if($this->saveTimelinePage($event->object)) {
				$this->redirect("./?page={$this->page->id}");
			}
		} elseif($this->input->post->action == 'save-footer-page') {
			if($this->saveFooterPage($event->object)) {
				$this->redirect("./?page={$this->page->id}");
			}
		} elseif($this->input->post->action == 'minres-install') {
			$this->installPage();
		}
	}

	/**
	 * Selector
	 * 
	 * When a "Create New Page" button is clicked, 
	 * the template selection should be displayed.
	 */
	public function ___renderSelector($event)
	{
		$this->page = ($event->object->page) ? $event->object->page : null;

		if($this->page || $this->input->post->action) return;
		// Not a new page or a template is already selected
		//if($this->page || $this->input->post->template || $this->input->post->action == 'select-template') return;

		$this->addHeaderResource('css', dirname($this->siteUrl).'/site/modules/minres-module/css/styles.css');

		ob_start(); ?>
		<h1><?php echo $this->i18n['select_template_header']; ?></h1>
		<form id="page-form" action="./<?php echo isset($this->input->get->page) ? 
			'?page='.(int)$this->input->get->page : ''; ?>" method="post">
			<?php 
			echo $this->renderTemplateSelectorField($event->object);
			echo $this->renderSelectorActionFields();
			?>
		</form>	
		<?php
		$event->return = ob_get_clean();
	}

	/**
	 * Selector
	 * 
	 * Renders TemplateSelector field
	 */
	protected function ___renderTemplateSelectorField($object)
	{
		// Templates
		$template_options = '';
		
		foreach($this->templates as $name => $template) {
			$pages = $object->pages->getItems("template=$name");
			$count = ($pages) ? count($pages) : 0;  
			$template_options .= '<option value="'.$name.'"'.
				(($name == $this->input->post->template) ? ' selected' : '').
				(($template['count'] > 0 && $template['count'] <= $count) ? ' disabled' : '').
				'>'.$name.'</option>';
		}
		ob_start(); ?>
		<div class="form-control">
			<div class="form-control">
				<p class="info-text i-wrapp"><i class="gg-danger"></i><span><?php echo 
					$this->i18n['template_selector_infotext'] ?></span></p>
			</div>
			<label for="template"><?php echo $this->i18n['template_label']; ?></label>
			<select name="template" id="template">
				<option><?php echo $this->i18n['parent_select_option']; ?></option>
				<?php echo $template_options; ?>
			</select>
		</div>
		<?php return ob_get_clean();
	}

	/**
	 * Selector actions fields
	 * 
	 *
	 */
	protected function ___renderSelectorActionFields()
	{
		ob_start() ?>
		<input type="hidden" name="action" value="select-template">
		<button class="icons" type="submit" id="save" name="save" value="1"><i class="gg-check"></i>
			<span>&nbsp;<?php echo $this->i18n['continue_button']; ?></span></button>
		<?php echo $this->csrf->renderInputs(); ?>
		<?php return ob_get_clean();
	}

	/**
	 * Editor
	 * 
	 * It's an entry point for all editors.
	 * 
	 * Executes: 
	 *   1. When a MinRes template is selected with Temp. Selector
	 *   2. The page already has a MinRes template.
	 * 
	 * @param object $event
	 */
	protected function ___renderEditorPage($event)
	{
		$this->page = ($event->object->page) ? $event->object->page : null;

		if($this->page && 
			!array_key_exists($this->page->template, $this->templates)
		) return;

		elseif(!$this->page && 
			!array_key_exists($this->input->post->template, $this->templates)
		) return;

		elseif(!$this->page && 
			$this->input->post->action == 'select-template' && 
			empty($this->input->post->template)
		) return;
		//if($event->return) return;
		
		$this->addHeaderResource('css', dirname($this->siteUrl).
			'/site/modules/minres-module/css/styles.css');
		$template = ($this->input->post->template) ? $this->input->post->template :
			$this->page->template;
		$method = 'render'.ucfirst($template).'Editor';

		ob_start(); ?>
		<!-- The Modal -->
		<div id="screen" class="modal">
			<!-- Modal content -->
			<div id="screen-content">
				<span class="close"><i class="gg-close"></i></span>
				<div id="page-text"></div>
			</div>
		</div>
		<h1><?php echo $this->i18n['edit_'.$template.'_header']; ?></h1>
		<form id="page-form" action="./<?php echo isset($this->input->get->page) ? 
			'?page='.(int)$this->input->get->page : ''; ?>" method="post">
			<?php 
			echo $this->$method();
			echo $this->renderEditorActionFields($template);
			?>
		</form>
		<?php 
		$this->jsConfig = $this->buildJsConfig([
			'allowHtmlOutput' => $this->config['allowHtmlOutput']
		]);
		$event->return = ob_get_clean();
	}

	/**
	 *
	 */
	protected function ___renderHeroEditor()
	{
		$data = ($this->page) ? 
			unserialize(base64_decode($this->page->content)) : [];

		ob_start(); ?>
		<div class="form-control t-grid">
			<div class="col-expand">
				<label  for="f-name"><?php echo $this->i18n['hero_firstname_label']; ?></label>
				<input name="first_name" id="f-name" type="text" value="<?php echo 
					isset($data['first_name']) ? $data['first_name'] : ''; ?>">
			</div>
			<div class="col-expand">
			<label for="l-name"><?php echo $this->i18n['hero_lastname_label']; ?></label>
				<input name="last_name" id="l-name" type="text" value="<?php echo 
					isset($data['last_name']) ? $data['last_name'] : ''; ?>">
			</div>
		</div>
		<div class="form-control">
			<label for="job-title"><?php echo $this->i18n['hero_job_title_label']; ?></label>
			<input name="job_title" id="job-title" type="text" value="<?php echo 
				isset($data['job_title']) ? $data['job_title'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="address"><?php echo $this->i18n['hero_address_label']; ?></label>
			<input name="address" id="address" type="text" value="<?php echo 
				isset($data['address']) ? $data['address'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="slogan"><?php echo $this->i18n['hero_slogan_label']; ?></label>
			<textarea class="minres-area" name="slogan" id="slogan"><?php echo 
				isset($data['slogan']) ? $data['slogan'] : ''; ?></textarea>
		</div>
		<input type="hidden" name="template" value="hero">
		<?php return ob_get_clean();
	}

	protected function ___renderSkillsEditor()
	{
		$data = ($this->page) ? 
			unserialize(base64_decode($this->page->content)) : [];

		ob_start(); ?>
		<div class="form-control">
			<label for="skills-label"><?php echo $this->i18n['skills_label_label']; ?></label>
			<input name="label" id="skills-label" type="text" value="<?php echo 
				isset($data['label']) ? $data['label'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="title"><?php echo $this->i18n['skills_title_label']; ?></label>
			<input name="title" id="title" type="text" value="<?php echo 
				isset($data['title']) ? $data['title'] : ''; ?>">
		</div>	
		<div class="form-control">
			<label for="exp"><?php echo $this->i18n['skills_exp_label']; ?></label>
			<input name="exp" id="exp" type="text" value="<?php echo 
				isset($data['exp']) ? $data['exp'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="text-info"><?php echo $this->i18n['skills_text_info_label']; ?></label>
			<input name="text_info" id="text-info" type="text" value="<?php echo 
				isset($data['text_info']) ? $data['text_info'] : ''; ?>">
		</div>
		<input type="hidden" name="template" value="skills">
		<?php return ob_get_clean();
	}

	protected function ___renderInterestsEditor()
	{
		$data = ($this->page) ? 
			unserialize(base64_decode($this->page->content)) : [];

		ob_start(); ?>
		<div class="form-control">
			<label for="interests-label"><?php echo $this->i18n['interests_label_label']; ?></label>
			<input name="label" id="interests-label" type="text" value="<?php echo 
				isset($data['label']) ? $data['label'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="title"><?php echo $this->i18n['interests_title_label']; ?></label>
			<input name="title" id="title" type="text" value="<?php echo 
				isset($data['title']) ? $data['title'] : ''; ?>">
		</div>	
		<div class="form-control">
			<label for="text-info"><?php echo $this->i18n['interests_text_info_label']; ?></label>
			<input name="text_info" id="text-info" type="text" value="<?php echo 
				isset($data['text_info']) ? $data['text_info'] : ''; ?>">
		</div>
		<input type="hidden" name="template" value="interests">
		<?php return ob_get_clean();
	}

	protected function ___renderAchievementsEditor()
	{
		$data = ($this->page) ? 
			unserialize(base64_decode($this->page->content)) : [];

		ob_start(); ?>
		<div class="form-control t-grid">
			<div class="col-expand">
				<label for="prefix"><?php echo $this->i18n['ach_prefix_label']; ?></label>
				<input name="prefix" id="prefix" type="text" value="<?php echo 
					isset($data['prefix']) ? $data['prefix'] : ''; ?>">
			</div>
			<div class="col-expand">
				<label for="number"><?php echo $this->i18n['ach_number_label']; ?></label>
				<input name="number" id="number" type="text" value="<?php echo 
					isset($data['number']) ? $data['number'] : ''; ?>">
			</div>
			<div class="col-expand">
				<label for="suffix"><?php echo $this->i18n['ach_suffix_label']; ?></label>
				<input name="suffix" id="suffix" type="text" value="<?php echo 
					isset($data['suffix']) ? $data['suffix'] : ''; ?>">
			</div>
		</div>
		<div class="form-control">
			<label for="title"><?php echo $this->i18n['interests_title_label']; ?></label>
			<input name="title" id="title" type="text" value="<?php echo 
				isset($data['title']) ? $data['title'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="ach-text"><?php echo $this->i18n['ach_text_label']; ?></label>
			<textarea class="minres-area" name="text" id="ach-text"><?php echo 
				isset($data['text']) ? $data['text'] : ''; ?></textarea>
		</div>
		<input type="hidden" name="template" value="achievements">
		<?php return ob_get_clean();
	}

	protected function ___renderTimelineEditor()
	{
		$data = ($this->page) ? 
			unserialize(base64_decode($this->page->content)) : [];

		ob_start(); ?>
		<div class="form-control">
			<label for="timeline-label"><?php echo $this->i18n['timeline_label_label']; ?></label>
			<input name="label" id="timeline-label" type="text" value="<?php echo 
				isset($data['label']) ? $data['label'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="title"><?php echo $this->i18n['timeline_title_label']; ?></label>
			<input name="title" id="title" type="text" value="<?php echo 
				isset($data['title']) ? $data['title'] : ''; ?>">
		</div>	
		<div class="form-control">
			<label for="text-info"><?php echo $this->i18n['timeline_text_info_label']; ?></label>
			<input name="period" id="text-info" type="text" value="<?php echo 
				isset($data['period']) ? $data['period'] : ''; ?>">
		</div>
		<input type="hidden" name="template" value="timeline">
		<?php return ob_get_clean();
	}

	protected function ___renderFooterEditor()
	{	
		$data = ($this->page) ? 
			unserialize(base64_decode($this->page->content)) : [];
		ob_start(); ?>
		<div class="form-control">
			<label for="title"><?php echo $this->i18n['footer_title_label']; ?></label>
			<input name="title" id="title" type="text" value="<?php echo 
				isset($data['title']) ? $data['title'] : ''; ?>">
		</div>
		<div class="form-control">
			<label for="ach-text"><?php echo $this->i18n['footer_text_label']; ?></label>
			<textarea class="minres-area" name="content" id="ach-text"><?php echo 
				isset($data['content']) ? $data['content'] : ''; ?></textarea>
		</div>
		<input type="hidden" name="template" value="footer">
		<?php return ob_get_clean();
	}

	protected function ___renderEditorActionFields($template)
	{
		$page = $this->page;

		ob_start()?>
		<input type="hidden" name="action" value="save-<?php echo $template ?>-page">
		<button class="icons" type="submit" id="save" name="save" value="1"><i class="gg-drive"></i>
			<span>&nbsp;<?php echo $this->i18n['save_button']; ?></span></button>
		<?php echo $this->csrf->renderInputs(); ?>
		<?php return ob_get_clean();
	}

	private function buildJsConfig(array $config) 
	{
		return '<script>const editConf = '.json_encode($config).'</script>';
	}

	// Todo: die Anzahl hero pages prüfen
	private function saveHeroPage($object)
	{
		// If not a single field is filled in, show error message.
		if(!$this->input->post->first_name &&
		!$this->input->post->last_name &&
		!$this->input->post->job_title &&
		!$this->input->post->address &&
		!$this->input->post->slogan) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_empty_fields']
			);
			return false;
		}

		$first_name = $this->sanitizer->text($this->input->post->first_name);
		$last_name = $this->sanitizer->text($this->input->post->last_name);
		$job_title = $this->sanitizer->text($this->input->post->job_title);
		$address = $this->sanitizer->text($this->input->post->address);
		$slogan = $this->sanitizer->textarea($this->input->post->slogan);

		$data = [
			'first_name' => $first_name,
			'last_name' => $last_name,
			'job_title' => $job_title,
			'address' => $address,
			'slogan' => $slogan
		];

		$this->page = null;

		if($this->input->get->page) {
			$this->page = $object->pages->getItem((int)$this->input->get->page);
		} else {
			$this->page = new Item($object->pages->id);
		}

		if(!$this->page) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_saving_minres_page']
			);
			return false;
		}

		if($this->config['protectCSRF'] && !$this->csrf->isTokenValid(
			$this->input->post->tokenName,
			$this->input->post->tokenValue, true)) {
				$this->msgs[] = array(
					'type' => 'error',
					'value' => Helper::rawClassName(__CLASS__).
						': '.$this->i18n['error_csrf_token_mismatch']
				);
			return false;
		}

		$this->page->set('parent', 1);
		$this->page->set('template', 'hero');
		$this->page->set('name', 'Hero');
		$this->page->set('slug', 'hero');
		$this->page->set('active', 1);
		$this->page->set('content', base64_encode(serialize($data)));
		$this->page->set('pagetype', 'minres');
		if(!$this->page->save()) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_saving_minres_page']
			);
			return false;
		}

		$this->msgs[] = array(
			'type' => 'success',
			'value' => Helper::rawClassName(__CLASS__).
				': '.$this->i18n['successful_saved_hero_page']
		);
		return true;
	}

	// Todo: die Anzahl hero pages prüfen
	private function saveSkillsPage($object)
	{
		// If not a single field is filled in, show error message.
		if(!$this->input->post->title &&
		!$this->input->post->exp &&
		!$this->input->post->text_info) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_empty_fields']
			);
			return false;
		}

		$label = $this->sanitizer->text($this->input->post->label);
		$title = $this->sanitizer->text($this->input->post->title);
		$exp = $this->sanitizer->text($this->input->post->exp);
		$text_info = $this->sanitizer->text($this->input->post->text_info);
		$template = $this->sanitizer->text($this->input->post->template);

		$data = [
			'label' => $label,
			'title' => $title,
			'exp' => $exp,
			'text_info' => $text_info,
			'template' => $template
		];

		return $this->savePage($data, $object->pages);
	}

	private function saveAchievementsPage($object)
	{
		if(!$this->input->post->title &&
			!$this->input->post->prefix &&
			!$this->input->post->suffix &&
			!$this->input->post->number &&
			!$this->input->post->text
		) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_empty_fields']
			);
			return false;
		}

		$title = $this->sanitizer->text($this->input->post->title);
		$prefix = $this->sanitizer->text($this->input->post->prefix);
		$suffix = $this->sanitizer->text($this->input->post->suffix);
		$number = $this->sanitizer->text($this->input->post->number);
		$text = $this->sanitizer->textarea($this->input->post->text);
		$template = $this->sanitizer->text($this->input->post->template);

		$data = [
			'title' => $title,
			'prefix' => $prefix,
			'suffix' => $suffix,
			'number' => $number,
			'text' => $text,
			'template' => $template
		];

		return $this->savePage($data, $object->pages);
	}

	private function saveTimelinePage($object)
	{
		if(!$this->input->post->title &&
			!$this->input->post->label &&
			!$this->input->post->period
		) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_empty_fields']
			);
			return false;
		}

		$title = $this->sanitizer->text($this->input->post->title);
		$label = $this->sanitizer->text($this->input->post->label);
		$period = $this->sanitizer->text($this->input->post->period);
		$template = $this->sanitizer->text($this->input->post->template);

		$data = [
			'title' => $title,
			'label' => $label,
			'period' => $period,
			'template' => $template
		];

		return $this->savePage($data, $object->pages);
	}

	private function saveFooterPage($object)
	{
		if(!$this->input->post->content) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_empty_fields']
			);
			return false;
		}

		$title = 'Footer';
		$content = htmlentities($this->input->post->content);
		$template = $this->sanitizer->text($this->input->post->template);

		$data = [
			'title' => $title,
			'content' => $content,
			'template' => $template
		];

		return $this->savePage($data, $object->pages);
	}


	private function savePage(array $fields, Category $pages)
	{
		if($this->config['protectCSRF'] && !$this->csrf->isTokenValid(
			$this->input->post->tokenName,
			$this->input->post->tokenValue, true)) {
				$this->msgs[] = array(
					'type' => 'error',
					'value' => Helper::rawClassName(__CLASS__).
						': '.$this->i18n['error_csrf_token_mismatch']
				);
			return false;
		}
		// Check template valid
		if(!array_key_exists($fields['template'], $this->templates)) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_minres_template_invalid']
			);
			return false;
		}

		$this->page = null;

		if($this->input->get->page) {
			$this->page = $pages->getItem((int)$this->input->get->page);
		} else {
			$this->page = new Item($pages->id);
		}

		if(!$this->page) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_saving_minres_page']
			);
			return false;
		}

		$this->page->set('parent', 1);
		$this->page->set('template', $fields['template']);
		$this->page->set('name', ucfirst($fields['template']).' - '.$fields['title']);
		$this->page->set('slug', $fields['title']);
		$this->page->set('active', 1);
		$this->page->set('content', base64_encode(serialize($fields)), false);
		$this->page->set('pagetype', 'minres');
		if(!$this->page->save()) {
			$this->msgs[] = array(
				'type' => 'error',
				'value' => Helper::rawClassName(__CLASS__).
					': '.$this->i18n['error_saving_minres_page']
			);
			return false;
		}

		$this->msgs[] = array(
			'type' => 'success',
			'value' => Helper::rawClassName(__CLASS__).
				': '.$this->i18n['successful_saved_'.$fields['template'].'_page']
		);
		return true;
	}

	protected function ___redirect($url)
	{
		Util::redirect($url);
	}

	private function renderInstaller($token)
	{
		ob_start(); ?>
		<!-- The Modal -->
		<h1><?php echo $this->i18n['minres_installer_header']; ?></h1>
		<script>
			var loaderText = '<?php echo $this->i18n['loader_minres_start_text'] ?>';
			const token = '<?php echo $token; ?>';
		</script>
		<script src="<?php echo dirname($this->siteUrl).'/site/modules/MinResModule/scripts/install.js' ?>"></script>
		<?php 
		/* $this->jsConfig = $this->buildJsConfig([
			'allowHtmlOutput' => $this->config['allowHtmlOutput']
		]); */
		return ob_get_clean();
	}

	private function installPage()
	{
		$data = include_once __DIR__.'/_default-pages.php';
		$index = (int) $this->input->post->index ?? 0;

		if ($index > 20) {
			unset($_SESSION['MinResToken']);
			$this->addMsg('error', Helper::rawClassName(__CLASS__).': '.$this->i18n['error_minres_install']);
			Helper::sendJsonResponse(['redirect' => 1]);
		}

		if (!isset($data) || empty($data)) {
			unset($_SESSION['MinResToken']);
			$this->addMsg('error', Helper::rawClassName(__CLASS__).': '.$this->i18n['error_minres_default_pages_data']);
			Helper::sendJsonResponse(['redirect' => 1]);
		}

		$token = $this->segments->getLast();
		if (!isset($_SESSION['MinRes']['token']) || $token !== $_SESSION['MinRes']['token']) {
			$this->msgs = $_SESSION['MinRes']['msgs'];
			unset($_SESSION['MinRes']);
			$this->addMsg('error', Helper::rawClassName(__CLASS__).': '.$this->i18n['error_minres_install']);
			Helper::sendJsonResponse(['redirect' => 1]);
		}

		foreach ($data as $key => $row) {
			$break = false;
			$exists = $this->site->pages()->getPages("slug=$row[slug]");
			if ($exists) {
				foreach ($exists as $p) {
					if ($p->pagetype == 'minres') { 
						$break = true;
						break;
					}
				}
			}
			if ($break) continue;

			$id = $this->site->pages()->category->id;
			$page = new Page($id);

			$page->set('parent', 1);
			$page->set('template', $row['template']);
			$page->set('name', $row['name']);
			$page->set('slug', $row['slug']);
			$page->set('active', 1);
			$page->set('content', $row['content']);
			$page->set('pagetype', 'minres');

			if (!$page->save()) {
				$this->msgs = $_SESSION['MinRes']['msgs'];
				unset($_SESSION['MinRes']);
				$this->addMsg('error', Helper::rawClassName(__CLASS__).': '.$this->i18n['error_saving_minres_page']);
				$this->sendJsonResponse(['redirect' => 1]);
			}

			sleep(1);

			if (isset($data[$key + 1])) {
				Helper::sendJsonResponse([ 
					'next' => 1, 
					'loaderText' => 'The <strong>'.$data[$key + 1]['name'].'</strong> page will be created...', 
					'index' => ++$index
				]);
			}

			$this->msgs = $_SESSION['MinRes']['msgs'];
			$this->addMsg('success', Helper::rawClassName(__CLASS__).': '.$this->i18n['successful_created_pages']);		
			unset($_SESSION['MinRes']);

			Helper::sendJsonResponse(['redirect' => 1]);
		}
	}
}